// ---------------------------------------------------------------------------
// [Filename here]
// 
// Written by:   
// Last Updated:
//
// Description
// This file has been provided 
// as the interface description for your homework.
//
// It should not require any alterations.
//
// ---------------------------------------------------------------------------
#pragma once

#include "DAExcept.h"    // for rangeError exception

using namespace std;

class DynArray
{
	public:
		
		// constructors
		// Postconditions: an empty vector with size = 5  and a specified capacity.
		DynArray(int capacity = 5);

		// copy constructor uses existing DynArray obj to create new DynArray(*this)
		// Postcondition: creates current DynArray (*this) as a copy of obj
		DynArray(const DynArray& obj);

		// destructor
		// Postcondition: the dynamic array is destroyed and memory returned to the heap
		~DynArray();

		// Precondition: the vector is not empty and index < size-1 
		// Postcondition: returns the value at index.
		int get(int index) const;

		// Precondition: 0 <= index < size-1 
		// Postcondition: update location index with value
		void set(int value, int index);

		//Postcondition: value appended at end of vector with size increased by 1
		void pushBack(int value);

		// insert value in nth index position of the vector.
		// Postcondition: value at index n with the rest of the elements moved up and size+1
		void insert(int value, int index);

		// remove element at index location in the vector.
		// Precondition: vector is not empty, else throws the underflowError exception
		// Postcondition: element at index removed and size--
		void remove(int index);

		// return current size of elements stored in Dynamic Array
		int size() const;

		// return true if vector is empty and false otherwise
		bool empty() const;

		// return the current capacity of the dynamic array
		int capacity() const;

		//print variable values and array for debugging
		void printVector() const;

		//operators 
		//Member function to overload the assignment operator
		DynArray& operator= (const DynArray& rhs);

		//Must make copy of dynamic array from rhs for *this (or left hand side of =)
		// compares twodynamic arrays for the same sizes and array values 
		friend bool operator== (const DynArray& lhs, const DynArray& rhs);

		// output the elements in vArr array in order.	
		friend ostream& operator<< (ostream& ostr, const DynArray& da);
			
   private:
		int m_capacity;		// amount of available space
		int m_size;			// number of elements in the list
		int* mp_array;		// the dynamic array

		// called by public functions only if size of array needs to exceed vCapacity. 
		// the vector capacity to n elements, copies the existing
		// elements to the new space if copy == true, and deletes
		// the old dynamic array. throws the memoryAllocationError
		// exception if memory allocation fails
		void growArray(int n, bool copy);
			
};
